/*
**  AUTO.C [edit EMAIL.H before compiling]
**
**  This program uses two channels concurrently
**  to automatically respond to all new email.
*/

#include <windows.h>
#include <stdio.h>
#include "see.h"
#include "email.h"
#include "keycode.h"

#define POP3_CHAN  0
#define SMTP_CHAN  1

#define BUFF_SIZE  2048
#define TEMP_SIZE   128

static char Buffer[BUFF_SIZE];
static char Reply[TEMP_SIZE];

void ShowError(int Chan, int Code)
{seeErrorText(Chan,Code,(LPSTR)Buffer,512);
 printf("SEE Error %d: %s (Chan %d)\n", Code, Buffer, Chan);
}

void ErrorExit(int Chan, int Code)
{ShowError(Chan, Code);
 seeClose(POP3_CHAN);
 seeClose(SMTP_CHAN);
 seeRelease();
 exit(1);
}

void StripCRLF(LPSTR Ptr)
{char c;
 while(1)
   {c = *Ptr;
    if(c=='\0') return;
    if((c=='\r')||(c=='\n'))
      {*Ptr = '\0';
       return;
      }
    Ptr++;
   }
}

void main(void)
{int i, n;
 int Code;
 int BufSize;
 int NbrMsg;
 int Version;
 LPSTR Ptr;
 char Address[40];
 /* attach SEE (2 channels) */
 Code = seeAttach(2, SEE_KEY_CODE);
 if(Code<0)
   {printf("Cannot attach SEE. Check SEE_KEY_CODE\n");
    exit(1);
   }
 Version = seeStatistics(POP3_CHAN, SEE_GET_VERSION);
 printf("SEE4C Version %1d.%1d.%1d\n",0x0f&(Version>>8),0x0f&(Version>>4),0x0f&Version);
 /* define diagnostics log file */
 seeStringParam(POP3_CHAN, SEE_LOG_FILE, (LPSTR)"auto-0.log");
 seeStringParam(SMTP_CHAN, SEE_LOG_FILE, (LPSTR)"auto-1.log");
 /* connect to POP3 server */
 printf("Connecting to POP3 server %s on channel %d\n", POP3_HOST_NAME, POP3_CHAN);
 Code = seePop3Connect(POP3_CHAN,
    (LPSTR)POP3_HOST_NAME,
    (LPSTR)POP3_USER_NAME,
    (LPSTR)POP3_PASSWORD);
 if(Code<0)
   {ShowError(POP3_CHAN, Code);
    seeRelease();
    exit(1);
   }
 /* connect to SMTP server */
 printf("Connecting to SMTP server %s on channel %d\n", SMTP_HOST_NAME, SMTP_CHAN);
 Code = seeSmtpConnect(SMTP_CHAN,
    (LPSTR)SMTP_HOST_NAME,           /* SMTP server */
    (LPSTR)YOUR_EMAIL_ADDR,          /* return email address */
    (LPSTR)YOUR_EMAIL_ADDR);         /* Reply-To header */
 if(Code<0)
   {ShowError(SMTP_CHAN, Code);
    seeClose(POP3_CHAN);
    seeRelease();
    exit(1);
   }
 /* get # messages waiting  */
 puts("Getting message count...");
 NbrMsg = seeGetEmailCount(POP3_CHAN);
 if(NbrMsg<0) ErrorExit(POP3_CHAN, NbrMsg);
 printf("%d messages waiting.\n", NbrMsg);
 /* read message headers */
 for(i=1;i<=NbrMsg;i++)
   {/* read message i */
    printf("Reading email message %d \n",i);
    /* read header */
    BufSize = seeGetEmailLines(POP3_CHAN, i, 0, (LPSTR)Buffer, BUFF_SIZE);
    if(BufSize<0) ErrorExit(POP3_CHAN, BufSize);
    Buffer[BufSize] = '\0';
    /* search for "Reply-To: " line first */
    n = seeExtractText((LPSTR)Buffer, "Reply-To: ", (LPSTR)Reply, TEMP_SIZE);
    if(n>0) Ptr = (LPSTR)Reply+10;
    else
      {/* search for "From:" line since no "Reply-To:" */
       n = seeExtractText((LPSTR)Buffer, "From: ", (LPSTR)Reply, TEMP_SIZE);
       Ptr = (LPSTR)Reply+6;
      }
    if(n>0)
      {StripCRLF(Ptr);
       printf("%s\n", (LPSTR)Reply);
       if(strchr(Ptr,'<')==NULL)
         {/* surround with angle brackets */
          strcpy(Address, "<");
          strcpy(Address, Ptr);
          strcat(Address, ">");
         }
       else strcpy(Address, Ptr);
       ///printf("[%s]\n", Address);
       /* send response */
       Code = seeSendEmail(SMTP_CHAN,
         (LPSTR)Address,
         (LPSTR)NULL,
         (LPSTR)NULL,
         (LPSTR)"Automated response",
         (LPSTR)"Your email has been received.",
         (LPSTR)NULL);
       if(Code<0) ErrorExit(SMTP_CHAN, Code);
       else printf("Email sent to %s\n", Address);
      }
   } /* end-for */
 /* close connection to servers */
 seeClose(POP3_CHAN);
 seeClose(SMTP_CHAN);
 seeDebug(-2, 0, NULL, 0);
 seeRelease();
 printf("Done.\n");
} /* end main */


