/*
**  MAILER.C [edit EMAIL.H before compiling]
**
**  This program emails a specified message.
*/

#define USE_SEE_DRIVER 1

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "see.h"
#include "keycode.h"
#include "email.h"

#define QUOTE 0x22

static char Buffer[512];

/* display error message */

static void ErrorExit(int Code, LPSTR Msg)
{printf("ERROR: ");
 if(Msg) printf("%s\n",Msg);
 if(Code)
   {seeErrorText(0,Code,(LPSTR)Buffer,50);
    printf("%s\n",(LPSTR)Buffer);
    seeRelease();
    exit(1);
   }
}

void main(int argc, char *argv[])
{int Code;
 int Version;
 LPSTR Attach = NULL;
 ULONG TimeMark;
 long BytesSent;
 if((argc!=4)&&(argc!=5))
   {printf("Usage: MAILER email-text subject <email-addr>\n");
    printf("Usage: MAILER email-text subject <email-addr> {attach-file}\n");
    printf("   eg: MAILER %c@test.mai%c %cThis is a test%c %c<mike@marshallsoft.com>%c\n",
                   QUOTE,QUOTE,QUOTE,QUOTE,QUOTE,QUOTE);
    exit(1);
   }

 /* display parameters */
 /* attach SEE */
 Code = seeAttach(1, SEE_KEY_CODE);
 if(Code<0) ErrorExit(Code,"seeAttach");
 Version = seeStatistics(0,SEE_GET_VERSION);
 printf("SEE4C Version %1d.%1d.%1d\n",0x0f&(Version>>8),0x0f&(Version>>4),0x0f&Version);
 printf("Server : %s\n",(LPSTR)SMTP_HOST_NAME);
 printf("  From : %s\n",(LPSTR)YOUR_EMAIL_ADDR);
 if(*argv[1]=='@') printf("  File : %s\n", argv[1]);
 else printf("  Text : %c%s%c\n", QUOTE,argv[1],QUOTE);
 printf("  Subj : %s\n", argv[2]);
 printf("    To : %s\n", argv[3]);
 if(argc>4)
   {Attach = argv[4];
    printf(" Attach: %s\n", Attach);
   }
 /* check format of email addresses */
 Code = seeVerifyFormat((LPSTR)argv[3]);
 if(Code<0) ErrorExit(Code,(LPSTR)argv[3]);
 Code = seeVerifyFormat(YOUR_EMAIL_ADDR);
 if(Code<0) ErrorExit(Code,(LPSTR)YOUR_EMAIL_ADDR);

 /* define diagnostics log file */
 seeStringParam(0,SEE_LOG_FILE, (LPSTR)"mailer.log");
 /* enable quoting */
 seeIntegerParam(0,SEE_QUOTED_PRINTABLE, QUOTED_PLAIN);
 ///seeIntegerParam(0,SEE_QUOTED_PRINTABLE, QUOTED_HTML);

 /* connect to SMTP server */
 printf("Connecting to SMTP server %s\n", SMTP_HOST_NAME);
 Code = seeSmtpConnect(0,
    (LPSTR)SMTP_HOST_NAME,           /* SMTP server */
    (LPSTR)YOUR_EMAIL_ADDR,          /* return email address */
    (LPSTR)YOUR_EMAIL_ADDR);         /* Reply-To header */
 if(Code<0)ErrorExit(Code,(LPSTR)"Connect:");

 /* turn off AUTO CALL driver */
 if(USE_SEE_DRIVER) seeIntegerParam(0,SEE_AUTO_CALL_DRIVER, 0);

 /* add our header lines */
 seeStringParam(0,SEE_SET_HEADER,
   (LPSTR)"X-Priority: Normal\r\nX-Sender: MarshallSoft Computing");

 /* send email */
 Code = seeSendEmail(0,
    (LPSTR)argv[3],                  /* To list */
    (LPSTR)NULL,                     /* CC list */
    (LPSTR)NULL,                     /* BCC list */
    (LPSTR)argv[2],                  /* subject */
    (LPSTR)argv[1],                  /* message text */
    Attach);                         /* MIME attachment */

 if(Code<0) ErrorExit(Code,(LPSTR)"Sending email:");

 if(USE_SEE_DRIVER)
   {puts("Sending mail...");
    /* run the driver to send the email */
    TimeMark = GetCurrentTime() + 500; /* WIN32 API function */
    while(1)
      {Code = seeDriver(0);
       if((GetCurrentTime()>=TimeMark)||(Code==0))
         {TimeMark = GetCurrentTime() + 500;
          /* display progress */
          BytesSent = (long) seeStatistics(0, SEE_GET_TOTAL_BYTES_SENT);
          printf("%ld bytes written.\r",BytesSent);
         }
       if(Code==0) break;
      }
    printf("\n");

    ///printf("Mail sent. (%d)\n", Code);

    /* turn AUTO CALL back on for seeClose */
    seeIntegerParam(0, SEE_AUTO_CALL_DRIVER, 1);

   }
 printf("Closing...\n");

 seeClose(0);
 seeRelease();
} /* end main */


