/*
**  QUICK.C
**
**  Simple program that sends a quick email! [edit EMAIL.H before compiling]
*/

#include <windows.h>
#include "see.h"
#include "keycode.h"
#include "message.h"
#include "paint.h"
#include "about.h"
#include "email.h"

#ifdef WIN32
#define USE_INS HINSTANCE
#define USE_PTR PSTR
#else
#define USE_INS HANDLE
#define USE_PTR LPSTR
#endif


LRESULT CALLBACK MainWndProc(HWND, UINT, WPARAM, LPARAM);

/* globals */

#define MAX_BUF 1024
#define MAX_STR   64

#define SET_TO    1
#define SET_SUBJ  2
#define SET_MSG   3

HWND hMainWnd;            /* main window handle */

static HMENU hMenu;
static USE_INS hInstance;
static int WinWidth = 8 * NCOLS;
static int WinHeight = 12 * NROWS + 48;
static HCURSOR ArrowCursor;
static HCURSOR WaitCursor;

static char Temp[MAX_BUF+16]  = "\0";
static char Server[MAX_STR+1] = SMTP_HOST_NAME;
static char From[MAX_STR+1]   = YOUR_EMAIL_ADDR;
static char To[MAX_STR+1]     = "\0";
static char Subj[MAX_STR+1]   = "\0";
static char Message[MAX_BUF+1]= "\0";

void ErrorMessage(LPSTR MsgPtr)
{
 MessageBox(hMainWnd,MsgPtr,"ERROR",MB_ICONEXCLAMATION | MB_OK);
}

void DisplayError(int Code)
{static char ErrorMsg[MAX_STR+1];
 Code = seeErrorText(0,Code,(LPSTR)ErrorMsg,MAX_STR);
 ///ErrorMsg[Code] = '\0';
 wsprintf((LPSTR)Temp,"SEE Error %d: %s", Code, (LPSTR)ErrorMsg);
 DisplayLine((LPSTR)Temp);
}

/* Dialog proc */

#ifdef WIN32
BOOL CALLBACK
#else
BOOL FAR PASCAL
#endif
ParamDlgProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam)
{int n;

 switch (iMsg)
   {
    case WM_INITDIALOG:
        return TRUE;
    case WM_COMMAND: 
       if(wParam==MSG_QUIT)
         {Subj[0] = '\0';
          To[0] = '\0';
          Message[0] = '\0';
          EndDialog(hDlg, TRUE);
          return TRUE;
         }
       if(wParam==MSG_SENDMAIL)
         {/* extract parameters */
          n = GetDlgItemText(hDlg, DB_SUBJ, (LPSTR)Subj, MAX_STR);            
          Subj[n] = '\0';
          n = GetDlgItemText(hDlg, DB_TO,  (LPSTR)To,  MAX_STR);           
          To[n] = '\0';           
          n = GetDlgItemText(hDlg, DB_MSG, (LPSTR)Message, MAX_BUF);            
          Message[n] = '\0';
          EndDialog(hDlg, TRUE);
          return TRUE;
         } 
    }
  return FALSE;
} /* end ParamDlgProc */

/* WinMain */

#ifdef WIN32
int WINAPI
#else
int PASCAL
#endif
WinMain(USE_INS hInst, USE_INS hPrevInstance,
        USE_PTR szCmdLine,  int nCmdShow)
{WNDCLASS  wc;
 MSG msg;
 BOOL Result;
 if(!hPrevInstance)
   {/* register main window class */
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = MainWndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInst;
    wc.hIcon = LoadIcon(hInst, "HostIcon");
    wc.hCursor = NULL;
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "HostMenu";
    wc.lpszClassName = "HostWClass";
    Result = RegisterClass(&wc);
    if(!Result) return FALSE;
   }

 /* create main window */
 hInstance = hInst;
 hMainWnd = CreateWindow(
        "HostWClass",   "QUICK",    WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT,  CW_USEDEFAULT,
        WinWidth,       WinHeight,
        NULL,           NULL,
        hInstance,      NULL);
 ShowWindow(hMainWnd, nCmdShow);
 UpdateWindow(hMainWnd);
 hMenu = GetMenu(hMainWnd);

 /* window control loop */

 while(GetMessage(&msg,NULL,0,0))
   {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
   }
 return (msg.wParam);
} /* end WinMain */

#ifdef WIN32
LRESULT CALLBACK
#else
long FAR PASCAL
#endif
MainWndProc(HWND hWindow,UINT iMsg,WPARAM wParam,LPARAM lParam)
{HDC hDC;
 PAINTSTRUCT ps;
 int Code;
#ifdef WIN32
#else
 static FARPROC lpfnAboutDlgProc;
 static FARPROC lpfnParamDlgProc;
#endif
 hMainWnd = hWindow;
 switch (iMsg)
    {case WM_CREATE:
      /* create cursors */
      ArrowCursor = LoadCursor(NULL, IDC_ARROW);
      WaitCursor = LoadCursor(NULL, IDC_WAIT);
      SetCursor(ArrowCursor);
#ifdef WIN32
#else
      /* create thunk for Win16 */
      lpfnAboutDlgProc = MakeProcInstance(AboutDlgProc,hInstance);
      lpfnParamDlgProc = MakeProcInstance(ParamDlgProc,hInstance);
#endif
      /* initialize paint module */
      PaintInit();
      DisplayLine((LPSTR)"Choose 'SendMail' above to set email fields & send email"); 
      DisplayLine((LPSTR)"Remember to enclose all email addresses in '<>' brackets");
      Code = seeAttach(1, SEE_KEY_CODE);
      if(Code<0) DisplayError(Code);
      break;

     case WM_COMMAND:
         switch(wParam)
           {case MSG_ABOUT :
#ifdef WIN32
               DialogBox(hInstance, "AboutBox", hMainWnd, AboutDlgProc);
#else
               DialogBox(hInstance, "AboutBox", hMainWnd, lpfnAboutDlgProc);
#endif
               return 0;
               
            case MSG_PARAM:
            
               /* get info from dialog box */
#ifdef WIN32
               DialogBox(hInstance, "ParamBox", hMainWnd, ParamDlgProc);
#else
               DialogBox(hInstance, "ParamBox", hMainWnd, lpfnParamDlgProc);
#endif
               /* check address */
               Code = seeVerifyFormat((LPSTR)To);           
               if(Code<0) 
                 {DisplayError(Code); 
                  break;
                 }  
               /* enable log file and display wait cursor */
               seeStringParam(0,SEE_LOG_FILE, (LPSTR)"quick.log"); 
               SetCursor(WaitCursor);
               /* connect to SMTP server */
               wsprintf((LPSTR)Temp,"Connecting to %s",(LPSTR)Server);
               DisplayLine((LPSTR)Temp);
               Code = seeSmtpConnect(0,
                  (LPSTR)Server,
                  (LPSTR)From, 
                  (LPSTR)NULL);
               if(Code<0) 
                  {DisplayError(Code); 
                   break;
                  }  
               /* send email */
               DisplayLine((LPSTR)"Sending mail...");
               Code = seeSendEmail(0,
                   (LPSTR)To,
                   (LPSTR)NULL,           
                   (LPSTR)NULL,       
                   (LPSTR)Subj,
                   (LPSTR)Message,
                   (LPSTR)NULL);                      
               if(Code<0)
                  {DisplayError(Code); 
                   break;
                  }      
               DisplayLine((LPSTR)"Closing...");
               DisplayLine((LPSTR)"Email has been sent.");
               SetCursor(ArrowCursor);
               seeClose(0);
               return 0; 
               
            case MSG_EXIT:
              seeRelease();
              DestroyWindow(hMainWnd);
              break;

            case MSG_DEBUG:
              break;
           }
         break;

    case WM_PAINT:
      HideCaret(hMainWnd);
      hDC = BeginPaint(hMainWnd, &ps);
      SetMapMode(hDC,MM_ANISOTROPIC);
      SelectObject(hDC, GetStockObject(OEM_FIXED_FONT) );
      PaintMain(hDC,&ps);
      EndPaint(hMainWnd,&ps);
      SetCaretPos(PaintGetColPos(),PaintGetRowPos());
      ShowCaret(hMainWnd);
      break;

    case WM_DESTROY:
      PostQuitMessage(0);
      break;

    default:
      return (DefWindowProc(hMainWnd, iMsg, wParam, lParam));
   }
 return 0;
}


