/*
**  READER.C [edit EMAIL.H before compiling]
**
**  This program reads the specified email message
**  and saves it to disk.
*/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "see.h"
#include "keycode.h"
#include "email.h"

#define QUOTE 0x22

static char Buffer[1024];

void ErrorExit(int Code)
{seeErrorText(0,Code,(LPSTR)Buffer,512);
 printf("SEE Error %d: %s\n", Code, Buffer);
 seeClose(0);
 seeRelease();
 exit(1);
}

void main(int argc, char *argv[])
{int i, Code;
 int Version;
 int Count;
 int MsgNbr;
 long MsgSize;
 long BytesRead = 0;
 if(argc!=3)
   {printf("Usage: READER MsgNbr Filename\n");
    printf("   eg: READER 1 message.mai\n");
    exit(1);
   }

 /* attach SEE */
 Code = seeAttach(1, SEE_KEY_CODE);
 if(Code<0) ErrorExit(Code);
 /* display parameters */
 Version = seeStatistics(0,SEE_GET_VERSION);
 printf("SEE4C Version %1d.%1d.%1d\n",0x0f&(Version>>8),0x0f&(Version>>4),0x0f&Version);
 printf("Server : %s\n",(LPSTR)POP3_HOST_NAME);
 printf("  User : %s\n",(LPSTR)POP3_USER_NAME);
 printf("  Pass : %s\n",(LPSTR)POP3_PASSWORD);
 printf("MsgNbr : %s\n",(LPSTR)argv[1] );
 printf("SaveTo : %s\n",(LPSTR)argv[2]);

 MsgNbr = atoi(argv[1]);
 if((MsgNbr<1)||(MsgNbr>1000))
   {printf("MsgNbr = %d out of range\n", MsgNbr);
    seeRelease();
    exit(1);
   }
 /* define diagnostics log file */
 seeStringParam(0, SEE_LOG_FILE, (LPSTR)"reader.log");
 /* prefix file names with "1-", "2-", etc for all attachments */
 seeIntegerParam(0, SEE_FILE_PREFIX, 1);
 /* decode unnamed attachments (if any) */
 seeIntegerParam(0, SEE_DECODE_UNNAMED, 1);
 /* connect to POP3 server */
 printf("Connecting to '%s'\n", (LPSTR)POP3_HOST_NAME);
 Code = seePop3Connect(0,
    (LPSTR)POP3_HOST_NAME,             /* POP3 server */
    (LPSTR)POP3_USER_NAME,             /* user */
    (LPSTR)POP3_PASSWORD);             /* Password */
 if(Code<0) ErrorExit(Code);
 /* get # messages waiting  */
 puts("Getting message count...");
 Count = seeGetEmailCount(0);
 if(Count<0) ErrorExit(Count);
 printf("%d messages waiting.\n", Count);
 /* read email message */
 if((Count>=1)&&(MsgNbr<=Count))
   {/* read message  */
    MsgSize = seeGetEmailSize(0,MsgNbr);
    if(MsgSize<0) ErrorExit((int)MsgSize);
    printf("Message %d has %ld bytes\n", MsgNbr, MsgSize);
    /* turn off AUTO CALL driver */
    seeIntegerParam(0,SEE_AUTO_CALL_DRIVER, 0);
    printf("Reading message %d...\n",MsgNbr);
    ///Code = seeGetEmailFile(0,MsgNbr, argv[2], "c:\\temp", "c:\\temp");
    Code = seeGetEmailFile(0,MsgNbr, argv[2], ".", ".");
    if(Code<0) ErrorExit(Code);
    /* run the driver */
    for(i=0;;i++)
      {Code = seeDriver(0);
       if(Code<0) ErrorExit(Code);
       if(((i%10)==9)||(Code==0))
         {/* display progress on last call & every 10th call to seeDriver() */
          BytesRead = (long) seeStatistics(0,SEE_GET_TOTAL_BYTES_READ);
          printf("%ld bytes read.\r",BytesRead);
         }
       if(Code==0) break;
       }
    printf("\n");
    /* turn AUTO CALL back on for seeClose */
    seeIntegerParam(0,SEE_AUTO_CALL_DRIVER, 1);
    if(Code<0) ErrorExit(Code);
    /* get list of attachment file names */
    Code = seeDebug(0, SEE_GET_ATTACH_NAMES, Buffer, 1024);
    if(Code>0)
      {printf("Attachment filenames are:\n");
       printf("%s\n", Buffer);
      }
   }
 else printf("No messages waiting\n");
 seeClose(0);
 seeRelease();
} /* end main */


