/*
**  THREAD.C
**
**  Starts multiple threads: Each checks a different POP3 account.
**  Edit the POP3 account settings (line 133) before compiling.
**
*/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <process.h>
#include "see.h"
#include "defines.h"
#include "keycode.h"

#define NBR_THREADS   10

#define QUOTE 0x22

typedef struct
{int  IsRunning;
 int  ThreadID;
 char Server[40];
 char User[40];
 char Pass[40];
} Pop3ParamType;

static Pop3ParamType Pop3Params[NBR_THREADS]; 

int CallTheDriver(int ThreadID)
{int Code;
 while(1)
   {Code = seeDriver(ThreadID); 
    if(Code<=0) break;     
    if(Code==9999)Sleep(50); 
   } 
 return Code;
}

/* POP3 thread function */

void Pop3Thread(PVOID DataPtr)
{int Code;
 int ThreadID;
 Pop3ParamType *ParamPtr;
 char Temp[40];
 ParamPtr = (Pop3ParamType *) DataPtr;
 ParamPtr->IsRunning = TRUE;
 /* starting POP3 thread to check email status */
 ThreadID = ParamPtr->ThreadID;
 printf("Thread %d: Started.\n",ThreadID);

 /* define diagnostics log file */
 sprintf(Temp,"THREAD%1d.LOG",ParamPtr->ThreadID); 
 seeStringParam(ThreadID,SEE_LOG_FILE, (LPSTR)Temp); 
 /* turn off AUTO CALL driver */
 seeIntegerParam(ThreadID, SEE_AUTO_CALL_DRIVER, 0);  
 /* connect to POP3 server */
 printf("Thread %d: Connecting to server %s for user %s\n", 
    ThreadID, (LPSTR)&(ParamPtr->Server),(LPSTR)&(ParamPtr->User) );
 Code = seePop3Connect(ThreadID,
    (LPSTR)&(ParamPtr->Server),    /* POP3 server */
    (LPSTR)&(ParamPtr->User),      /* user */ 
    (LPSTR)&(ParamPtr->Pass));     /* Password */
Code = CallTheDriver(ThreadID);
 if(Code<0)
    {printf("Thread %d: Cannot connect to %s\n", 
       ThreadID,(LPSTR)&(ParamPtr->Server));
     ParamPtr->IsRunning = FALSE;
     return;  
   } 
 ///printf("Thread %d: Getting email count\n", ThreadID);  
 /* get # messages waiting  */
 Code = seeGetEmailCount(ThreadID); 
 if(Code<0)
   {printf("Thread %d: Error checking mailbox\n",ThreadID);
    ParamPtr->IsRunning = FALSE;
    seeClose(ThreadID);
    return;
   }
 
 CallTheDriver(ThreadID); 
 Code = seeStatistics(ThreadID, SEE_GET_MSG_COUNT);
 
 printf("Thread %d: %d messages waiting for %s on %s\n", 
    ThreadID, Code, (LPSTR)&(ParamPtr->User), (LPSTR)&(ParamPtr->Server) );
 seeClose(ThreadID);
 CallTheDriver(ThreadID);
 /* thread has completed */
 ParamPtr->IsRunning = FALSE;
 printf("Thread %d: Completed.\n", ThreadID);
}

void AddUser(int Index, LPSTR Server, LPSTR User, LPSTR Pass)
{strcpy((LPSTR)(&Pop3Params[Index].Server), Server);
 strcpy((LPSTR)(&Pop3Params[Index].User),   User); 
 strcpy((LPSTR)(&Pop3Params[Index].Pass),   Pass);
 Pop3Params[Index].IsRunning = FALSE;
 Pop3Params[Index].ThreadID  = Index;
}

/*** MAIN ***/

void main(int argc, char *argv[])
{int i;
 int Code;
 int Version;
 int Running;
 unsigned long TimeStart;
 unsigned long TimeStop;
 int ThreadsToRun;
 
 if(argc!=1)
   {printf("Usage: THREAD\n");
    exit(1);
   } 
   
 /* hard code one POP3 setting per thread  */
 /* 'ThreadsToRun' must be <= 'NBR_THEADS' */
 
 AddUser(0, "10.0.0.1", "mike",    "mike");
 AddUser(1, "10.0.0.1", "pam",     "pam");
 
 AddUser(2, "10.0.0.2", "mike",    "mike");
 AddUser(3, "10.0.0.2", "pam",     "pam");
 AddUser(4, "10.0.0.2", "lauren",  "lauren");
 AddUser(5, "10.0.0.2", "bingo",   "bingo");
 
 ThreadsToRun = 6;
 
 //AddUser(6, "10.0.0.4", "mike",    "mike");
 //AddUser(7, "10.0.0.4", "pam",     "pam");
 //AddUser(8, "10.0.0.4", "lauren",  "lauren");
 //AddUser(9, "10.0.0.4", "bingo",   "bingo");
 
 //ThreadToRun = 10;
 
 /* allocate channels (one per thread) */
 Code = seeAttach(ThreadsToRun, SEE_KEY_CODE);
 if(Code<0) 
   {if(Code==SEE_BAD_KEY_CODE) printf("%d is wrong key\n",SEE_KEY_CODE);
    else printf("ERROR: cannot seeAttach fails (%d)\n", Code);
    exit(1);
   }

 /* display SEE parameters */
 Version = seeStatistics(0,SEE_GET_VERSION);
 printf("SEE4C Version %1d.%1d.%1d\n",0x0f&(Version>>8),0x0f&(Version>>4),0x0f&Version); 
 
 /* set internal SEE SleepTime value (default = 20) */
 seeIntegerParam(0, SEE_SLEEP_TIME, 100);
 
 /* start POP3 threads */
 for(i=0;i<ThreadsToRun;i++) 
  {
   _beginthread(Pop3Thread, 0, (void *)&Pop3Params[i] );
  }
 Sleep(100);
 /* get start start time */
 TimeStart = GetCurrentTime();
 
 /* run until all threads are done */
 while(1)
   {/* are all threads done ? */
    Running = 0;
    for(i=0;i<ThreadsToRun;i++) if(Pop3Params[i].IsRunning) Running++;
    if(Running==0)
      {TimeStop = GetCurrentTime();
       printf("All threads have terminated.\n");
       printf("Total time = %d mS per user\n", (TimeStop - TimeStart) / ThreadsToRun);
       seeRelease();
       return;
      }
    Sleep(0);
   }

} /* end main */

