#include "stdafx.h"
#include "DialogWrapper.h"
#include "Atlbase.h"
#include <comdef.h>

#define MAX_BUFFER 256
#define MAX_TEXTBUF 2048

CDialogWrapper::CDialogWrapper(UINT nID):
m_hParent(NULL),
m_hDlg(NULL),
m_pQTWrapper(NULL)
{
	m_nID = nID;
	// loads strings from resource
	LoadString(IDS_DIALOGTITLE, m_strDialogCaption);
	LoadString(IDS_BUTTON1_TITLE, m_strQuickTagCtrlCaption);
	LoadString(IDS_BUTTON2_TITLE, m_strVersionCtrlCaption);
	LoadString(IDS_EDIT1_START, m_strInitialPhrases);
	// loads icon from resource
	m_hIcon = ::LoadIcon((HINSTANCE)::GetModuleHandle(NULL),MAKEINTRESOURCE(IDI_QUICKTAGICON));
}

CDialogWrapper::~CDialogWrapper()
{
	// destroys icon
	if(m_hIcon)
	{
		::DestroyIcon(m_hIcon);
		m_hIcon = NULL;
	}
	if(m_pQTWrapper)
	{
		m_pQTWrapper->Release();
	}
}

// Call this member function to invoke the modal dialog box and return the dialog-box result when done. 
int CDialogWrapper::DoModal(HWND hParent)
{
	if (hParent)
		m_hParent = hParent;
	else
		m_hParent = GetDesktopWindow();

	return (DialogBoxParam((HINSTANCE)::GetModuleHandle(NULL), MAKEINTRESOURCE(m_nID), m_hParent,
			DlgProc, (LPARAM)this));
}

// Call this member function to invoke the modeless dialog box  and return the dialog-box handle when displayed.
HWND CDialogWrapper::DoModeless(HWND hParent)
{
	if (hParent)
		m_hParent = hParent;
	else
		m_hParent = GetDesktopWindow();
	
	return(CreateDialogParam((HINSTANCE)::GetModuleHandle(NULL), MAKEINTRESOURCE(m_nID), m_hParent,
			DlgProc, (LPARAM)this));
}

BOOL CALLBACK CDialogWrapper::DlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	CDialogWrapper* pThis = (CDialogWrapper*)::GetWindowLong(hDlg, DWL_USER);
	try
	{
		if(uMsg == WM_INITDIALOG)
		{
			pThis = (CDialogWrapper*)lParam;
			pThis->m_hDlg = hDlg;
			SetWindowLong(hDlg, DWL_USER, lParam);
			if(!pThis->Initialize())
			{
				::EndDialog(hDlg, 0);
				return FALSE;
			}
		}
		else if (!pThis)
		{
			return(FALSE);
		}
		LRESULT lResult = 0;
		return pThis->ProcessDlgMessage(uMsg, wParam, lParam, lResult);
	}
	catch(...)
	{
	}
	return FALSE;
}

// execute "QuickTag"  function
LRESULT CDialogWrapper::OnQTButtonClick(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{	
	if(!m_pQTWrapper)
	{
		return FALSE;
	}

	// read strings from multiline editbox
	std::string strText = "";
	int nLine = ::SendMessage(::GetDlgItem(m_hDlg, IDC_EDIT1), EM_GETLINECOUNT, 0, 0);
	for(int i = 0; i < nLine; i++)
	{
		char* szText = new char[MAX_BUFFER];
		::FillMemory((void*)szText, MAX_BUFFER, ' ');
		LRESULT lResult = ::SendMessage(::GetDlgItem(m_hDlg, IDC_EDIT1), EM_GETLINE, (WPARAM)i, (LPARAM)szText);
		if(lResult > 0)
		{
			char* szTemp = new char[lResult + 2];
			::FillMemory((void*)szTemp, lResult, ' ');
			szTemp[lResult] = '\n';
			szTemp[lResult + 1] = '\0';
			strncpy(szTemp, szText, lResult);
			strText += szTemp;
			delete szTemp;
		}
		delete szText;
	}

	if(!strText.length())
	{
		m_strVersionCtrlCaption = "";
		SetCtrlCaption(m_strVersionCtrlCaption, IDC_EDIT2);
		return FALSE;
	}

	// execute function "QuickTag"
	BSTR bstrText = ::SysAllocString(T2BSTR(strText.c_str()));
	BSTR bstrOutFile = ::SysAllocString(T2BSTR(QTFileName));
	HRESULT hr;
	hr = m_pQTWrapper->QuickTag(bstrText, bstrOutFile);
	if(SUCCEEDED(hr))
	{
		if(hr == S_FALSE)
			MessageBox(IDS_ERR_EXECOPERATION);
		else
		{
			ReadFile();
			SetCtrlCaption(m_strVersionCtrlCaption, IDC_EDIT2);
		}
	}
	return TRUE;
}

// execute "GetQuickTagVersion" function
LRESULT CDialogWrapper::OnVersionButtonClick(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{	
	if(!m_pQTWrapper)
	{
		return FALSE;
	}
	BSTR bstrVersion = NULL;
	HRESULT hr;
	hr = m_pQTWrapper->GetQuickTagVersion(&bstrVersion);
	if(SUCCEEDED(hr))
	{
		_bstr_t szVersion(bstrVersion);
		m_strVersionCtrlCaption = (char*)szVersion;
		SetCtrlCaption(m_strVersionCtrlCaption, IDC_EDIT2);
		return TRUE;
	}
	return FALSE;
}

LRESULT CDialogWrapper::OnClose(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	::EndDialog(m_hDlg, 0);
	return 1;
}

BOOL CDialogWrapper::Initialize()
{
	if(!m_hDlg)
		return FALSE;

	SetCtrlCaption(m_strDialogCaption);
	SetCtrlCaption(m_strQuickTagCtrlCaption, IDC_BUTTON1);
	SetCtrlCaption(m_strVersionCtrlCaption, IDC_BUTTON2);
	SetCtrlCaption(m_strInitialPhrases, IDC_EDIT1);
	if(m_hIcon)
	{
		::SendMessage(m_hDlg, WM_SETICON, ICON_BIG, (LPARAM)m_hIcon);
	}

	HRESULT hr;
	hr = ::CoCreateInstance(CLSID_QTWrapper, NULL, CLSCTX_ALL, IID_IQTWrapper,(LPVOID*) &m_pQTWrapper);
	if((FAILED(hr)) || !m_pQTWrapper)
		return FALSE;

	// prepare data for init quicktag.dll
	std::string strFullDatFileName;
	std::string strDatFileName;
	char* szCurDir = new char[MAX_PATH];
	::ZeroMemory((void*)szCurDir, MAX_PATH);
	// load file name from resource for initializing of "QuickTag"
	LoadString(IDS_DATFILENAME, strDatFileName);
	if(!::GetCurrentDirectory(MAX_PATH, szCurDir))
		return FALSE;
	strFullDatFileName = szCurDir;
	strFullDatFileName += "\\";
	strFullDatFileName += strDatFileName;
	delete szCurDir;
	// execute function "InitQuickTag"
	CComBSTR bstrDataFile(strFullDatFileName.c_str());
	hr = m_pQTWrapper->InitQuickTag(bstrDataFile);
	if(FAILED(hr))
		return FALSE;
	return TRUE;
}

void CDialogWrapper::LoadString(UINT nIDString, std::string& str)
{
	char* szBuf = new char[MAX_BUFFER];
	::ZeroMemory((void*)szBuf, MAX_BUFFER);
	str = "";
	if(::LoadString((HINSTANCE)::GetModuleHandle(NULL), nIDString, szBuf, MAX_BUFFER))
	{
		str = szBuf;
	}
	delete szBuf;
}

BOOL CDialogWrapper::SetCtrlCaption(std::string strCaption, UINT nID)
{
	if(!m_hDlg)
		return FALSE;
	HWND hwndCtrl;
	hwndCtrl = (nID != 0) ? ::GetDlgItem(m_hDlg, nID) : m_hDlg;
	return ::SetWindowText(hwndCtrl, strCaption.c_str());
}

void CDialogWrapper::ReadFile()
{
	FILE* pFile = NULL;
	pFile = fopen(QTFileName, "r");
	if(pFile != NULL)
	{
		char* szText = new char[MAX_PATH];
		::ZeroMemory((void*)szText, MAX_PATH);
		m_strVersionCtrlCaption = "";
		while ((fgets(szText, MAX_PATH, pFile)) != NULL)
		{
			m_strVersionCtrlCaption += szText;
			m_strVersionCtrlCaption[m_strVersionCtrlCaption.length() - 1] = '\r';
			m_strVersionCtrlCaption += '\n';
		}
		delete szText;
		fclose(pFile);
	}
}

void CDialogWrapper::MessageBox(UINT nID)
{
	std::string strText;
	std::string strCaption;
	LoadString(nID, strText);
	LoadString(IDS_MESSAGEBOXWARNING, strCaption);
	::MessageBox(m_hDlg, strText.c_str(), strCaption.c_str(), MB_OK | MB_ICONWARNING);
}