#ifndef _DIALOGWRAPPER_H
#define _DIALOGWRAPPER_H

#include "resource.h"
#include <string>
#include <commctrl.h>

#define QTFileName "QTTest.txt"
//////////////////////////////////////////////////////////////////////////////////////
// defines macroses for message handlers
#define BEGIN_MSG_MAP() \
public: \
	BOOL ProcessDlgMessage(UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT& lResult, DWORD dwMsgMapID = 0) \
	{ \
		BOOL bHandled = TRUE; \
		switch(dwMsgMapID) \
		{ \
		case 0:

#define END_MSG_MAP() \
			break; \
		default: \
			break; \
		} \
		return FALSE; \
	}

#define MESSAGE_HANDLER(msg, func) \
	if(uMsg == msg) \
	{ \
		bHandled = TRUE; \
		lResult = func(uMsg, wParam, lParam, bHandled); \
		if(bHandled) \
			return TRUE; \
	}

#define COMMAND_ID_HANDLER(id, func) \
	if(uMsg == WM_COMMAND && id == LOWORD(wParam)) \
	{ \
		bHandled = TRUE; \
		lResult = func(HIWORD(wParam), LOWORD(wParam), (HWND)lParam, bHandled); \
		if(bHandled) \
			return TRUE; \
	}

#define NOTIFY_HANDLER(id, cd, func) \
	if(uMsg == WM_NOTIFY && id == ((LPNMHDR)lParam)->idFrom && cd == ((LPNMHDR)lParam)->code) \
	{ \
		bHandled = TRUE; \
		lResult = func((int)wParam, (LPNMHDR)lParam, bHandled); \
		if(bHandled) \
			return TRUE; \
	}

/////////////////////////////////////////////////////////////////////////////////////
// Declaration non-MFC wrapper class for dialog
class CDialogWrapper
{
public:
	CDialogWrapper(UINT nID);
	virtual ~CDialogWrapper();

	int DoModal(HWND hParent = NULL);
	HWND DoModeless(HWND hParent = NULL);

protected:
	// dialog procedure
	static BOOL CALLBACK DlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

BEGIN_MSG_MAP()
	COMMAND_ID_HANDLER(IDC_BUTTON1, OnQTButtonClick)
	COMMAND_ID_HANDLER(IDC_BUTTON2, OnVersionButtonClick)
	MESSAGE_HANDLER(WM_CLOSE, OnClose)
END_MSG_MAP()

// message handler for getting version
LRESULT OnVersionButtonClick(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
// message handler for executing QuickTag
LRESULT OnQTButtonClick(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
// message handler for closing dialog
LRESULT OnClose(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);

private:
	// loads string from resource - nIDString - ID of resource's string
	void LoadString(UINT nIDString, std::string& str);
	// sets text into child window
	BOOL SetCtrlCaption(std::string strCaption, UINT nID = 0);
	// read text from file
	void ReadFile();
	// display warning
	void MessageBox(UINT nID);

	// inializes the dialog
	BOOL Initialize();

	HICON	m_hIcon; // dialog's icon
	HWND	m_hParent; // the parent window
	HWND	m_hDlg; // the dialog window
	UINT	m_nID; // resource Id of dialog
	HINSTANCE	m_hQuickTagInstance; // instance of QuickTag.dll
	// strings for child controls
	std::string	m_strDialogCaption;
	std::string	m_strQuickTagCtrlCaption;
	std::string	m_strVersionCtrlCaption;
	std::string	m_strInitialPhrases;
};

// typedef for GetQuickTagVersion function
typedef void (WINAPI* DLLGETVERSIONPROC)(char*);
// typedef for InitQuickTag function
typedef int (WINAPI* DLLINITQUICKTAG)(char*);
// typedef for QuickTag function
typedef int (WINAPI* DLLQUICKTAG)(char*, char*);

#endif // _DIALOGWRAPPER_H
